#-----------------------------------------------------
# Scene_Simulation
# 戦闘シミュレーションを行うシーンクラス
# 2019.06.28 レベルの廃止、常にLv.1に
#-----------------------------------------------------
class Scene_Simulation < Scene_Base
  # 初期化
  def initialize(*ext)
    @enemy_stack = 0
    @image_enemy = nil
    @sprite_enemy = nil
    create_image
    super
    @position = 0
    @wear = 0
    @pant = 0
    create_background
  end
  # 全てのウィンドウを作成
  def create_all_windows
    super
    create_enemyhelp_window
    create_help_window
    create_enemy_list
    create_narrow_window
    create_battle_bk
    create_setbk_window
    create_operation_guide
  end
  # ヘルプウィンドウの作成
  def create_enemyhelp_window
    @windows[:enemyhelp] = Window_EnemyHelp.new(25, 579, 1233, 135)
  end
  # 絞込み内容ウィンドウを作成
  def create_help_window
    @windows[:help] = Window_Help.new(25, 573, 1233, 135)
  end
  # 敵リストを作成
  def create_enemy_list
    @windows[:enemy] = Window_EnemyList.new(65,116,404,343)
    @windows[:enemy].set_handler(:ok, method(:battle_ok))
    @windows[:enemy].set_handler(:cancel, method(:battle_cancel))
    @windows[:enemy].set_handler(:favorite, method(:recollect_favorite))
    @windows[:enemy].set_handler(:narrow, method(:recollect_narrow))
    @windows[:enemy].set_handler(:battlebk, method(:recollect_bk))
    @windows[:enemy].set_handler(:bgm, method(:recollect_bgm)) if $system.game_clear
    @windows[:enemy].enemy_image(@sprite_enemy, @image_enemy)
    @windows[:enemy].help_window = @windows[:enemyhelp]
    @windows[:enemy].up_down_img_xy(270,112,270,458)
    @windows[:enemy].display_up_down = true
    @windows[:enemy].scene = :battle
    @windows[:enemy].active
    @windows[:enemy].refresh
  end
  # 絞り込みウィンドウを作成
  def create_narrow_window
    @windows[:narrow] = Window_Narrow.new(65,123,480,332)
    @windows[:narrow].set_handler(:cancel, method(:narrow_cancel))
    @windows[:narrow].help_window = @windows[:help]
    @windows[:narrow].visible = false
    @windows[:narrow].up_down_img_xy(270,112,270,458)
    @windows[:narrow].display_up_down = true
    @windows[:narrow].deactive
    @windows[:narrow].refresh
  end
  # 戦闘背景設定ウィンドウを作成
  def create_setbk_window
    @windows[:setbk] = Window_SetBattleBK.new(65,116,404,343)
    @windows[:setbk].set_handler(:ok, method(:setbk_ok))
    @windows[:setbk].set_handler(:cancel, method(:setbk_cancel))
    @windows[:setbk].help_window = @windows[:help]
    @windows[:setbk].visible = false
    @windows[:setbk].up_down_img_xy(270,112,270,458)
    @windows[:setbk].display_up_down = true
    @windows[:setbk].select($system.battle_bk)
    @windows[:setbk].set_bk_sprite(@sprite_battle_bk)
    @windows[:setbk].refresh
    @windows[:setbk].deactive
  end
  # 操作説明
  def create_operation_guide
    @guide = Sprite_Base.new(65, 490)
    @guide.image = Image_Message.new(1233, 32)
    @guide.image.default_size = 32
    unless $system.game_clear
      @guide.image.draw_text(0, 0, Vocab::BattleSimuGuide)
    else
      @guide.image.draw_text(0, 0, Vocab::BattleSimuGuide + "　Wキー：サウンドテスト起動")
    end
  end
  #キャラのイメージを作成
  def create_image
    @sprite_enemy = Sprite_Base.new(608.5, 126)
    @image_enemy = Pis.load($game_directory + '/data/img/menu/enemies.pis')
  end
  # 戦闘背景スプライトを作成
  def create_battle_bk
    @sprite_battle_bk = Sprite_Base.new(609, 126)
    @sprite_battle_bk.center_x = 0
    @sprite_battle_bk.center_y = 0
    @sprite_battle_bk.scale_x = 0.95
    @sprite_battle_bk.scale_y = 0.95
  end
  # 背景を作成
  def create_background
    @bg= Sprite_Pis.new("data/img/menu/simulation.pis", 0, 0)
    @sprite_enemy.z = @bg.z + 2
    @sprite_battle_bk.z = @bg.z + 2
  end
  # 戦闘＞キャンセル
  def battle_cancel
    SceneManager.return
  end
  # 戦闘へ移行
  def goto_battle
    ScreenShot.capture
    VEffect.start_draw_loading if $system.battle_setplugin == 2
    @enemy_stack = @windows[:enemy].index
    BattleManager.setup(@windows[:enemy].item)
    BattleManager.set_simulation
    SceneManager.call($system.battle_setplugin == 2 ? Scene_Battle : Scene_Plugin)
  end
  # 敵リスト(戦闘)の決定
  def battle_ok
    @windows[:enemy].deactive
    @windows[:enemy].active
    goto_battle
  end
  # お気に入り
  def recollect_favorite
    if $system.favo_recollect.include?(@windows[:enemy].item)
      Audio.play_cancel
      $system.favo_recollect.delete(@windows[:enemy].item)
      @windows[:enemy].refresh
    else
      Audio.play_ok
      $system.favo_recollect << @windows[:enemy].item
      @windows[:enemy].refresh
    end
  end
  # 絞り込みウィンドウ
  def recollect_narrow
    Audio.play_ok
    @windows[:enemy].visible = false
    @windows[:enemyhelp].visible = false
    @windows[:enemy].deactive
    @windows[:narrow].visible = true
    @windows[:help].visible = true
    @windows[:narrow].active
  end
  # 絞り込みウィンドウキャンセル
  def narrow_cancel
    Audio.play_cancel
    @windows[:enemy].refresh
    @windows[:narrow].visible = false
    @windows[:help].visible = false
    @windows[:narrow].deactive
    @windows[:enemy].visible = true
    @windows[:enemyhelp].visible = true
    @windows[:enemy].active
  end
  # サウンドテスト起動
  def recollect_bgm
    Audio.play_ok
    SceneManager.call(Scene_Jukebox)
  end
  # 戦闘背景設定ウィンドウ
  def recollect_bk
    Audio.play_ok
    @windows[:enemy].visible = false
    @sprite_enemy.visible = false
    @windows[:enemyhelp].visible = false
    @windows[:enemy].deactive
    @windows[:setbk].visible = true
    @windows[:help].visible = true
    @windows[:setbk].active
  end
  # 戦闘背景設定
  def setbk_ok
    Audio.play_ok
    $system.battle_bk = @windows[:setbk].item
    @windows[:setbk].refresh
  end
  # 戦闘背景設定ウィンドウキャンセル
  def setbk_cancel
    Audio.play_cancel
    @windows[:enemy].refresh
    @windows[:setbk].visible = false
    @windows[:help].visible = false
    @windows[:setbk].deactive
    @windows[:enemy].visible = true
    @sprite_enemy.visible = true
    @windows[:enemyhelp].visible = true
    @windows[:enemy].active
    @windows[:setbk].select($system.battle_bk)
  end
  # 更新
  def update
    @sprite_enemy.draw if !@sprite_enemy.nil?
    @sprite_battle_bk.draw if @windows[:setbk].visible?
    @bg.update
    @bg.draw
    @guide.update
    @guide.draw
    super
  end
  # 戦闘開始時、キックする関数のダミー
  def spriteset
    return nil
  end
  # 戦闘終了後、画像の再定義（関数名は通常のバトル終了時にキックする関数名準拠）
  def load_events_image_stacks
    create_image
    create_all_windows
    @windows[:enemy].select(@enemy_stack)
  end
  # 戦闘終了後、キックする関数のダミー #
  def spriteset_load_event
  end
  def trancefer_sprites_co_by_pxy
  end
  # ---------------------------------#
  # 解放処理
  def dispose
    sprite_dispose
  end
  # 画像の解放処理
  def sprite_dispose
    @windows.each{|key, window| window.dispose }
    @windows = nil
    @sprite_enemy.image.dispose rescue nil
    @sprite_enemy.dispose
    @sprite_enemy.vanish
    @image_enemy.each{|enemy| enemy.dispose}
    @sprite_battle_bk.dispose
    @bg.dispose
    @guide.image.dispose
    @guide.dispose
    @guide.vanish
    @image_enemy = nil
    @sprite_enemy = nil
    @bg = nil
    @guide = nil
  end
end
