#-----------------------------------------------------
# スクリプトファイルを管理するためのモジュール。
#-----------------------------------------------------

module RequireList
  # ゲーム初期化
  def self.init
    require 'dxruby'
    require 'fileutils'
    require 'logger'
    self.load_modules
    self.load_classes
  end
  # モジュールロード用処理
  def self.load_modules
    require_relative './module Const'
    require_relative './iniapp'
    require_relative './module Key'
    require_relative './module SceneManager'
    require_relative './module Vocab'
    require_relative './module Audio'
    require_relative './module Fonts'
    require_relative './module EventLoader'
    require_relative './module DataManager'
    require_relative './module VEffect'
    require_relative './module ExpTable'
    require_relative './module ScreenShot'
    require_relative './module BattleManager'
    require_relative './module Data_Item'
    require_relative './module Data_Plugin'
    require_relative './module Data_Recipe'
    require_relative './module Data_Skill'
    require_relative './module Data_State'
    require_relative './module Data_MapPos'
    require_relative './module Data_SubMission'
    require_relative './module Data_Onom'
    require_relative './module Data_Enemy'
    require_relative './module Data_EnemyGraphic'
    require_relative './module Data_EnemyMessage'
    require_relative './module Data_EnemyPosition'
    require_relative './module Data_EnemyRecollect'
    require_relative './module Data_Tutorial'
    require_relative './module Data_Pants'
    require_relative './module Data_Trophy'
    require_relative './module Data_Chapter'
    require_relative './module Data_MusicList'
    require_relative './module Data_Album'
    require_relative './module Data_BattleBK'
    require_relative './module Unique_Event'

  end
  # クラスロード用処理
  def self.load_classes
    require_relative './class Line'
    require_relative './class Bezier'
    require_relative './class SingleCurve'
    require_relative './Data_Map'
    require_relative './class Map'
    require_relative './class Image'
    require_relative './class Image_Message'
    require_relative './class Actor'
    require_relative './class Enemy'
    require_relative './class Item_Base'
    require_relative './class Item_Use'
    require_relative './class Item_Plugin'
    require_relative './class Item_Skill'
    require_relative './class Item_Plugin'
    require_relative './class Item_Debuff'
    require_relative './class Item_State'
    require_relative './class Item_Mission'
    require_relative './class Buff'
    require_relative './class Action'
    require_relative './class ActionResult'
    require_relative './class Audio_Data'
    require_relative './class Game_Player'
    require_relative './class Game_System'
    require_relative './class String'
    require_relative './class Sprite_Base'
    require_relative './class Sprite_Animation'
    require_relative './class Sprite_Pis'
    require_relative './class Sprite_Actor'
    require_relative './class Sprite_Player'
    require_relative './class Sprite_MenuBack'
    require_relative './class Sprite_MenuStatus'
    require_relative './class Sprite_State'
    require_relative './class Sprite_BattleState'
    require_relative './class Sprite_EventState'
    require_relative './class Sprite_Place'
    require_relative './class Sprite_Point'
    require_relative './class Sprite_PointEp'
    require_relative './class Sprite_PointLevel'
    require_relative './class Sprite_Meter'
    require_relative './class Sprite_Balloon'
    require_relative './class Sprite_BattleBalloon'
    require_relative './class Sprite_Effect'
    require_relative './class Sprite_Weather'
    require_relative './class Sprite_OneLoop'
    require_relative './class Sprite_MessageLog'
    require_relative './class Sprite_MessageMode'
    require_relative './class Sprite_Motion'
    require_relative './class Sprite_Success'
    require_relative './class Sprite_EndCredit'
    require_relative './class Pis'
    require_relative './class Spriteset_Map'
    require_relative './class Sprite_MapName'
    require_relative './class Spriteset_Battle'
    require_relative './class Spriteset_Enemy'
    require_relative './class Spriteset_MenuCommand'
    require_relative './class Spriteset_Slot'
    require_relative './class Spriteset_MenuStatus'
    require_relative './class Spriteset_MapBack'
    require_relative './class Spriteset_Motions'
    require_relative './class Spriteset_ReleaseCommand'
    require_relative './class Cursor'
    require_relative './class Rect'
    require_relative './class Diamond'
    require_relative './class Gauge_Horz'
    require_relative './class Gauge_Vert'
    require_relative './class Gauge_Hp'
    require_relative './class Gauge_HpEvent'
    require_relative './class Gauge_EnemyHp'
    require_relative './class Gauge_Sp'
    require_relative './class Gauge_SpEvent'
    require_relative './class Gauge_VertSp'
    require_relative './class Point_sp'
    require_relative './class HLA'
    require_relative './class Plugin_HLA'
    require_relative './class Window_Origin'
    require_relative './class Window_Selectable'
    require_relative './class Window_Command'
    require_relative './class Window_HorzCommand'
    require_relative './class Window_Message'
    require_relative './class Window_Choices'
    require_relative './class Window_BattleCommand'
    require_relative './class Window_BattleMessage'
    require_relative './class Window_BattleChoices'
    require_relative './class Window_MenuCommand'
    require_relative './class Window_PluginCommand'
    require_relative './class Window_Help'
    require_relative './class Window_EnemyHelp'
    require_relative './class Window_SubMission'
    require_relative './class Window_CGorEVENT'
    require_relative './class Window_EnemyList'
    require_relative './class Window_Narrow'
    require_relative './class Window_CG'
    require_relative './class Window_ItemCategory'
    require_relative './class Window_MenuItem'
    require_relative './class Window_BattleItem'
    require_relative './class Window_PluginItem'
    require_relative './class Window_PluginNarrow'
    require_relative './class Window_PluginMaterial'
    require_relative './class Window_PluginComp'
    require_relative './class Window_Preset'
    require_relative './class Window_PresetPlugin'
    require_relative './class Window_PresetOperation'
    require_relative './class Window_MapList'
    require_relative './class Window_Slot'
    require_relative './class Window_BattleLog'
    require_relative './class Window_MenuSkill'
    require_relative './class Window_BattleSkill'
    require_relative './class Window_PluginStatus'
    require_relative './class Window_Option'
    require_relative './class Window_MenuConfig'
    require_relative './class Window_SetBattleBK'
    require_relative './class Window_KeyConfig'
    require_relative './class Window_DataCategory'
    require_relative './class Window_MenuDatabase'
    require_relative './class Window_MenuEnemyStatus'
    require_relative './class Window_MenuEnemyDatabase'
    require_relative './class Window_MenuPantsDatabase'
    require_relative './class Window_ChapterCategory'
    require_relative './class Window_Chapter'
    require_relative './class Window_Summary'
    require_relative './class Window_Title'
    require_relative './class Window_Mode'
    require_relative './class Window_File'
    require_relative './class Window_Load'
    require_relative './class Window_SaveExit'
    require_relative './class Window_Save'
    require_relative './class Window_Exit'
    require_relative './class Window_Level'
    require_relative './class Window_MusicList'
    require_relative './class Window_Album'
    require_relative './class Window_EndCredit'
    require_relative './class Scene_Base'
    require_relative './class Scene_Title'
    require_relative './class Scene_Map'
    require_relative './class Scene_Menu'
    require_relative './class Scene_Plugin'
    require_relative './class Scene_Mission'
    require_relative './class Scene_Load'
    require_relative './class Scene_Battle'
    require_relative './class Scene_Recollect'
    require_relative './class Scene_CG'
    require_relative './class Scene_RecollectRoom'
    require_relative './class Scene_Simulation'
    require_relative './class Scene_MapSelect'
    require_relative './class Scene_Jukebox'
    require_relative './class Scene_Album'
    require_relative './class Scene_EndCredit'
    require_relative './class Sound'
    require_relative './class Sound_Bgm'
    require_relative './class Sound_SE'
    require_relative './class Sound_Moan'
    require_relative './class Event'
    require_relative './class Event_Page'
    require_relative './class Event_Command'
    require_relative './class Battle_Events'
    require_relative './class Battle_EventCommand'
    require_relative './class Battler'
    #最終確認項目######################################
    require_relative './class Scene_SwitchOperater'
    require_relative './class Window_SwitchSection'
    require_relative './class Window_SwitchSeparate'
    ##################################################
  end
end
