#-----------------------------------------------------
# module VEffect
# 画面のフェードアウト・フェードインを行う
#-----------------------------------------------------
module VEffect
  # 初期化
  def self.init
    @fade_duration = 0
    @fade_mode = :none
    @image = {}
    @image[:black] = Image.new(Const::AppWidth, Const::AppHeight, C_BLACK)
    @image[:white] = Image.new(Const::AppWidth, Const::AppHeight, C_WHITE)
    @image[:hoe]   = Image.new(Const::AppWidth, Const::AppHeight, C_DEFAULT)
    # フェードアウト用スプライト
    @sprite = Sprite_Base.new(0, 0)
    @sprite.z = 105
    @sprite.alpha = 0
    # 色調変化用スプライト
    @sprite_hoe = Sprite_Base.new(0, 0)
    @sprite_hoe.z = 90
    @sprite_hoe.alpha = 0
    @sprite_hoe.image = @image[:hoe]
    @hoe_duration = [255, 0, 0, 0, 0, 0] # [0~3]:各色相の変化幅 [4]:残りフレーム数 [5]:スプライトの透明度
    # フラッシュ用スプライト
    @sprite_flash = Sprite_Base.new(0, 0)
    @sprite_flash.z = 95
    @sprite_flash.alpha = 0
    @sprite_flash.image = @image[:white]
    @flash_duration = 0
    @flash_mx_alpha = 255
    # ローディング画面用スプライト
    @sprite_loading = Sprite_Base.new(0, 0)
    @sprite_loading.z = 10000
    @sprite_loading.image = Image.load($game_directory + '/data/img/loading.png')
    @sprite_loading.scale_x = 1280.0 / @sprite_loading.image.width.to_f
    @sprite_loading.scale_y = 720.0 / @sprite_loading.image.height.to_f
    @sprite_loading.center_x = 0
    @sprite_loading.center_y = 0
    @on_draw_loading = false
    # ピクチャ表示用スプライト
    @sprite_picture = Sprite_Base.new(0, 0)
    @sprite_picture.z = 100
    @sprite_picture.alpha = 0
    @sprite_picture.x = Const::AppWidth / 2
    @sprite_picture.y = Const::AppHeight / 2
    @sprite_picture.offset_sync = true
    @sprite_picture.angle = 0
    @picture_duration = {duration: 0, opacity: 0}
    @picture_move = {duration: 0, x: @sprite_picture.x, y: @sprite_picture.y}
    @picture_angle = {duration: 0, angle: 0}
    # エフェクト用スプライト
    @sprite_effect = Sprite_Effect.new(0, 0)
    @sprite_effect.z = 89
    @sprite_effect.x = Const::AppWidth / 2
    @sprite_effect.y = Const::AppHeight / 2
    @sprite_effect.offset_sync = true
    # 天候エフェクト用スプライト
    @sprite_weather = Sprite_Weather.new
    @sprite_weather.z = 91
    # 画像データのプリロード
    init_balloon
    # フキダシ画像の画面同期用変数
    @balloon_sx = 0
    @balloon_sy = 0
    @balloon_ox = 0
    @balloon_oy = 0
    @effect_ox = 0
    @effect_oy = 0
    @sync_type = :none
    # ミニゲーム用成功／失敗画像
    @success_image = Sprite_Success.new
    @success_image.z = 101
    @success_image_count = BattleManager::RELEASE_RESULT_COUNT
  end
  # フキダシの初期化
  def self.init_balloon
    @sprites_balloon = []
    $image_balloon = []
    $image_balloon[1] = Pis.load($game_directory + "/data/img/balloon/anger.pis")
    $image_balloon[2] = Pis.load($game_directory + "/data/img/balloon/guchagucha.pis")
    $image_balloon[3] = Pis.load($game_directory + "/data/img/balloon/heart.pis")
    $image_balloon[4] = Pis.load($game_directory + "/data/img/balloon/note1.pis")
    $image_balloon[5] = Pis.load($game_directory + "/data/img/balloon/note2.pis")
    $image_balloon[6] = Pis.load($game_directory + "/data/img/balloon/question.pis")
    $image_balloon[7] = Pis.load($game_directory + "/data/img/balloon/skull.pis")
    $image_balloon[8] = Pis.load($game_directory + "/data/img/balloon/surprise_red.pis")
    $image_balloon[9] = Pis.load($game_directory + "/data/img/balloon/surprise_yellow.pis")
    $image_balloon[10] = Pis.load($game_directory + "/data/img/balloon/sweat.pis")
    $image_balloon[11] = Pis.load($game_directory + "/data/img/balloon/swirl.pis")
    $image_balloon[12] = Pis.load($game_directory + "/data/img/balloon/tear.pis")
    $image_balloon[13] = Pis.load($game_directory + "/data/img/balloon/silence.pis")
    $image_balloon[14] = Pis.load($game_directory + "/data/img/balloon/light.pis")
  end
  # 更新
  def self.update
    self.process_fade
    self.process_hoe
    self.process_flash
    self.process_picture
    self.update_balloons
    self.update_success
    @sprite.update
    @sprite_hoe.update
    @sprite_flash.update
    @sprite_picture.update
    @sprite.draw
    @sprite_hoe.draw
    @sprite_flash.draw
    @sprite_picture.draw
    @sprite_effect.update
    @sprite_weather.update if SceneManager.scene.instance_of?(Scene_Map)
    @success_image.update
    if @on_draw_loading
      @sprite_loading.update
      @sprite_loading.draw
    end
  end
  # フェードの処理
  def self.process_fade
    case @fade_mode
    when :out
      @sprite.alpha = [@sprite.alpha + @fade_duration, 0].max
    when :in
      @sprite.alpha = [@sprite.alpha - @fade_duration, 255].min
    when :end
      @fade_mode = :none
    end
    self.stop_fade if (@sprite.alpha <= 0 || @sprite.alpha >= 255) && @fade_mode != :none
  end
  # フェードの停止処理
  def self.stop_fade
    case @fade_mode
    when :out
      @sprite.alpha = 255
    when :in
      @sprite.alpha = 0
    end
    @fade_mode = :end
    @fade_duration = 0
    # @sprite.alpha = [[0, @sprite.alpha].max, 255].min
  end
  # 色調の処理
  def self.process_hoe
    if self.changing_hoe?
      color = Array.new(4)
      pipette = @image[:hoe][0, 0]
      pipette.each_index{|index|
        color[index] = [[0, pipette[index] - @hoe_duration[index]].max, 255].min
      }
      @image[:hoe].fill(color)
      @sprite_hoe.alpha = @sprite_hoe.alpha - @hoe_duration[5]
      @hoe_duration[4] = @hoe_duration[4] - 1
    end
  end
  # フラッシュの処理
  def self.process_flash
    if self.flashing?
      @flash_duration -= 1
      @sprite_flash.alpha = @flash_rate + @sprite_flash.alpha
      if @sprite_flash.alpha < 0 || @sprite_flash.alpha > @flash_mx_alpha
        @sprite_flash.alpha = [[0, @sprite_flash.alpha].max, @flash_mx_alpha].min
        @flash_rate = - @flash_rate
      end
    else
      @sprite_flash.alpha = 0
      @flash_rate = 0
    end
  end
  # ピクチャの処理
  def self.process_picture
    # フェードの処理
    if @picture_duration[:duration] == 0
      @sprite_picture.alpha = @picture_duration[:opacity]
    else
      step = (@sprite_picture.alpha - @picture_duration[:opacity]) / @picture_duration[:duration]
      @sprite_picture.alpha = @sprite_picture.alpha - step
      @picture_duration[:duration] -= 1
    end
    # ピクチャ移動の処理
    if @picture_move[:duration] == 0
      @sprite_picture.x = @picture_move[:x]
      @sprite_picture.y = @picture_move[:y]
    else
      step_x = (@sprite_picture.x - @picture_move[:x]) / @picture_move[:duration]
      step_y = (@sprite_picture.y - @picture_move[:y]) / @picture_move[:duration]
      @sprite_picture.x = @sprite_picture.x - step_x
      @sprite_picture.y = @sprite_picture.y - step_y
      @picture_move[:duration] -= 1
    end
    # ピクチャ回転の処理
    if @picture_angle[:duration] == 0
      @sprite_picture.angle = @picture_angle[:angle]
    else
      step = (@sprite_picture.angle - @picture_angle[:angle]) / @picture_angle[:duration]
      @sprite_picture.angle = @sprite_picture.angle - step
      @picture_angle[:duration] -= 1
    end
  end
  # ピクチャの表示フェード中か？
  def self.picture_infade?
    return @picture_duration[:duration] > 0
  end
  # エフェクトの画面同期処理(シェイク)
  def self.effect_shake(x, y)
    @sprite_effect.x = @effect_ox + x
    @sprite_effect.y = @effect_oy + y
  end
  # エフェクトの画面同期処理(スクロール)
  def self.effect_scroll(x, y)
    @sprite_effect.x = @sprite_effect.x + x
    @sprite_effect.y = @sprite_effect.y + y
  end
  # フキダシ画像の更新処理
  def self.update_balloons
    sp = []
    @sprites_balloon.each{|sprite|
      if sprite.active?
        case @sync_type
        when :shake
          sprite.x = @balloon_ox + @balloon_sx
          sprite.y = @balloon_oy + @balloon_sy
        when :scroll
          sprite.x = sprite.x + @balloon_sx
          sprite.y = sprite.y + @balloon_sy
        end
        sprite.update
      else
        sprite.dispose
        sprite.vanish
        sp << sprite
      end
    }
    sp.each{|disposed_sprite|
      @sprites_balloon.delete(disposed_sprite)
    }
  end
  # フキダシ画像の画面同期処理(基本処理)
  def self.balloon_sync(x, y)
    @balloon_sx = x
    @balloon_sy = y
  end
  # フキダシ画像の画面同期処理(シェイク)
  def self.balloon_shake(x, y)
    balloon_sync(x, y)
    @sync_type = :shake
  end
  # フキダシ画像の画面同期処理(スクロール)
  def self.balloon_scroll(x, y)
    balloon_sync(x, y)
    @sync_type = :scroll
  end
  # フェードアウトの開始
  def self.fadeout(color, duration)
    return if @fade_mode == :out
    if duration == 0
      self.stop_fade
      return
    end
    @fade_mode = :out
    @sprite.image = @image[color]
    @fade_duration = 256 / duration.to_f
  end
  # フェードインの開始
  def self.fadein(duration)
    return if @fade_mode == :in
    if duration == 0
      @sprite.alpha = 255
      return
    end
    @fade_mode = :in
    @fade_duration = (256 / duration).to_i
  end
  # 画面色調の変更
  def self.change_hoe(color, depth, duration)
    palette = Const::Palette[color]
    if duration == 0
      @image[:hoe].fill(palette)
      @sprite_hoe.alpha = [[0, depth].max, 255].min
    else
      pipette = @image[:hoe][0, 0]
      pipette.each_index{|index|
        @hoe_duration[index] = (pipette[index] - Const::Palette[color][index]) / duration
      }
      @hoe_duration[4] = duration.to_f
      @hoe_duration[5] = (@sprite_hoe.alpha - depth) / duration.to_f
    end
  end
  # フラッシュの開始
  def self.flash(rate, duration, alpha=255)
    @flash_rate = rate
    @flash_duration = duration
    @flash_mx_alpha = alpha
  end
  # フェードのリセット
  def self.reset_fade_stat
    @fade_mode = :none
    @fade_duration = 0
    @sprite.alpha = 0
  end
  # フェードがリセットされているか？
  def self.reseted_fade?
    return :none == @fade_mode
  end
  # フェードアウト中か？
  def self.in_fadeout?
    return :out == @fade_mode
  end
  # フェードイン中か？
  def self.in_fadein?
    return :in == @fade_mode
  end
  # フェードが終了しているか？
  def self.fin_fade?
    return :end == @fade_mode
  end
  # 色調変化の処理中か？
  def self.changing_hoe?
    return @hoe_duration[4] != 0
  end
  # フラッシュの処理中か？
  def self.flashing?
    return @flash_duration != 0
  end
  # フキダシの処理中か？
  def self.balloons_active?
    @sprites_balloon.each{|sprite| return true if sprite.active?}
    return false
  end
  # ロード画面の表示
  def self.start_draw_loading
    @on_draw_loading = true
  end
  # ロード画面の非表示
  def self.end_draw_loading
    @on_draw_loading = false
  end
  # ロード画面表示中か？
  def self.draw_loading?
    return @on_draw_loading
  end
  # ピクチャのロード
  def self.load_picture(data)
    if @sprite_picture.image.instance_of?(Image)
      @sprite_picture.image.dispose
      @sprite_picture.image = nil
    end
    if data.instance_of?(Image)
      @sprite_picture.image = data
    elsif data.instance_of?(String)
      if data.include?("pis")
        pis = Pis.load(data)
        @sprite_picture.image = pis[0]
      else
        @sprite_picture.image = Image.load(data)
      end
    end
    @sprite_picture.center_x = @sprite_picture.image.width / 2
    @sprite_picture.center_y = @sprite_picture.image.height / 2
  end
  # ピクチャの表示
  def self.show_picture(duration = 0, opacity = 255)
    @picture_duration[:duration] = duration
    @picture_duration[:opacity] = opacity
  end
  # ピクチャの非表示
  def self.hide_picture(duration = 0)
    @picture_duration[:duration] = duration
    @picture_duration[:opacity] = 0
  end
  # ピクチャの位置設定
  def self.move_picture(x, y, duration)
    @picture_move[:duration] = duration
    @picture_move[:x] = x
    @picture_move[:y] = y
  end
  # ピクチャのz軸設定
  def self.z_picture(z)
    @sprite_picture.z = z
  end
  # ピクチャの回転設定
  def self.rotate_picture(angle, duration)
    @picture_angle[:duration] = duration
    @picture_angle[:angle] = angle
  end
  # ピクチャの消去
  def self.delete_picture
    return if !@sprite_picture.image.instance_of?(Image)
    @sprite_picture.image.dispose
    @sprite_picture.image = nil
    @sprite_picture.angle = 0
    @sprite_picture.z = 100
    @sprite_picture.center_x = 0
    @sprite_picture.center_y = 0
  end
  # バルーンの表示
  def self.show_balloon(id, x, y, length = :normal)
    @sprites_balloon << Sprite_Balloon.new(id, x, y, length)
    @balloon_ox = x
    @balloon_oy = y
    @sync_type = :none
  end
  # エフェクトの表示
  def self.uprise_effect(symbol, x, y, scale, angle, times, hls=[0,0,0], alpha=255)
    @sprite_effect.clear_instances
    @sprite_effect.set_effect(symbol, x, y, scale, angle, times, hls, alpha)
    @effect_ox = x
    @effect_oy = y
  end
  # 天候エフェクトの表示
  def self.uprise_weather(symbol)
    # 天候に変化が無い場合は何もしない
    unless self.now_weather_chk(symbol)
      @sprite_weather.clear_instances
      #晴れの場合は天候エフェクトを削除して終了
      @sprite_weather.set_effect(symbol) unless symbol == :fine
    end
  end
  # 設定される天候と今の天候に変化が無いか
  def self.now_weather_chk(symbol)
    @sprite_weather.now_weather? == symbol
  end
  # ミニゲームの成功／失敗
  def self.minigame_success(bool)
    @success_image.success(bool)
  end
  # ミニゲームの成功／失敗画像更新処理
  def self.update_success
    if @success_image.active?
      @success_image_count -= 1
      if 55 > @success_image_count && @success_image_count > 5
        @success_image.success_image_x = 1
      else
        @success_image.success_image_x = 752.5 / 5
      end
      if @success_image_count <= 0
        @success_image.co_clear
        @success_image_count = BattleManager::RELEASE_RESULT_COUNT
      end
    end
  end
  # フラッシュの処理中か？
  def self.success_displaying?
    return @success_image_count != BattleManager::RELEASE_RESULT_COUNT
  end
  # エフェクトの表示中か？
  def self.in_effect?
    @sprite_effect.active?
  end
end
